from typing import List

from dataclass_csv import DataclassReader

from ynab_import.common.providers import CSVProvider
from ynab_import.swedbank.models import SwedbankTransaction


class SwedbankCSVProvider(CSVProvider):
    CLIENT_ACCOUNT = "Client account"
    DATE = "Date"
    PAYEE = "Beneficiary/Payer"
    MEMO = "Details"
    AMOUNT = "Amount"
    DEBIT_CREDIT = "Debit/Credit"

    def read_csv(self, input_file: str) -> List[SwedbankTransaction]:
        with open(input_file, "r", encoding="utf8") as f:
            reader = DataclassReader(
                f, SwedbankTransaction, delimiter=";", quotechar='"'
            )
            reader.map(self.CLIENT_ACCOUNT).to("client_account")
            reader.map(self.DATE).to("date")
            reader.map(self.PAYEE).to("payee")
            reader.map(self.MEMO).to("memo")
            reader.map(self.AMOUNT).to("amount")
            reader.map(self.DEBIT_CREDIT).to("debit_credit")
            return list(reader)



from environs import Env

env = Env()
env.read_env()

INPUT_FILE_PATH = None
OUTPUT_FILE_PATH = None

SOURCE = env.str("SOURCE")
READ_FROM = env.str("READ_FROM")
WRITE_TO = env.str("WRITE_TO")

# Ynab Config
YNAB_API_BASE_URL = "https://api.youneedabudget.com/v1/"
YNAB_API_KEY = env.str("YNAB_API_KEY")
YNAB_BUDGET_ID = env.str("YNAB_BUDGET_ID")
YNAB_ACCOUNT_ID = env.str("YNAB_ACCOUNT_ID")



from dataclasses import replace
from typing import List

from ynab_import.common.models import YnabTransaction
from ynab_import.common.transformers import Transformer
from ynab_import.swedbank.models import SwedbankTransaction


class SwedBankTransformer(Transformer):
    """Data transformation From Swedbank To Ynab"""

    SKIP_KEYWORDS = ["closing balance", "Turnover", "Opening balance"]

    def prepare_data(self, swedbank: SwedbankTransaction) -> SwedbankTransaction:
        ynab_amount = self.ynab_amount(swedbank.amount)
        if swedbank.debit_credit == "D":
            ynab_amount = str(ynab_amount * -1)
        return replace(swedbank, amount=ynab_amount)

    def to_ynab_transactions(
        self, transactions: List[SwedbankTransaction]
    ) -> List[YnabTransaction]:
        res = []
        for tr in transactions:
            if tr.memo in self.SKIP_KEYWORDS:
                continue
            tr = self.prepare_data(tr)
            import_id = self.generate_import_id(
                self.account_id, tr.amount, tr.date.isoformat(),
            )
            res.append(
                YnabTransaction(
                    account_id=self.account_id,
                    date=tr.date.isoformat(),
                    amount=tr.amount,
                    payee_name=tr.payee,
                    memo=tr.memo,
                    cleared="cleared",
                    approved=False,
                    flag_color="red",
                    import_id=import_id,
                )
            )
        return res



from typing import List

from dataclass_csv import DataclassReader

from ynab_import.common.providers import CSVProvider
from ynab_import.revolut.models import RevolutTransaction


class RevolutCSVProvider(CSVProvider):
    COMPLETED_DATE = "Completed Date "
    DESCRIPTION = " Description "
    PAID_OUT_EUR = " Paid Out (EUR) "
    PAID_IN_EUR = " Paid In (EUR) "
    EXCHANGE_OUT = " Exchange Out"
    EXCHANGE_IN = " Exchange In"
    BALANCE_EUR = " Balance (EUR)"
    CATEGORY = " Category"
    NOTES = " Notes"

    def read_csv(self, input_file: str) -> List[RevolutTransaction]:
        with open(input_file, "r", encoding="utf8") as f:
            reader = DataclassReader(
                f, RevolutTransaction, delimiter=";", quotechar='"'
            )
            reader.map(self.COMPLETED_DATE).to("completed_date")
            reader.map(self.DESCRIPTION).to("description")
            reader.map(self.PAID_OUT_EUR).to("paid_out_eur")
            reader.map(self.PAID_IN_EUR).to("paid_in_eur")
            reader.map(self.EXCHANGE_OUT).to("exchange_out")
            reader.map(self.EXCHANGE_IN).to("exchange_in")
            reader.map(self.BALANCE_EUR).to("balance_eur")
            reader.map(self.CATEGORY).to("category")
            reader.map(self.NOTES).to("notes")
            return list(reader)



import click

from ynab_import.revolut.providers import RevolutCSVProvider
from ynab_import.revolut.transformers import RevolutTransformer
from ynab_import.swedbank.providers import SwedbankCSVProvider
from ynab_import.swedbank.transformers import SwedBankTransformer
from ynab_import.ynab.providers import YnabBudget


@click.command()
@click.option("--source", envvar="SOURCE", type=click.Choice(["swedbank", "revolut"]))
@click.option("--read-from", envvar="READ_FROM")
@click.option(
    "--write-to", envvar="WRITE_TO", default="api", type=click.Choice(["stdout", "api"])
)
@click.option("--account-id", envvar="YNAB_ACCOUNT_ID")
@click.option("--budget-id", envvar="YNAB_BUDGET_ID")
@click.option("--api-key", envvar="YNAB_API_KEY")
def main(source, read_from, write_to, account_id, budget_id, api_key):
    # default csv swedbank
    provider = SwedbankCSVProvider()
    if source == "revolut" and read_from != "api":
        provider = RevolutCSVProvider()

    # read
    data = provider.read_csv(input_file=read_from)

    # transform
    transformer = SwedBankTransformer(account_id=account_id)
    if source == "revolut":
        transformer = RevolutTransformer(account_id=account_id)

    data = transformer.to_ynab_transactions(data)

    if write_to == "stdout":
        for i in data:
            print(i)
    elif write_to == "api":
        yn = YnabBudget(api_key=api_key, budget_id=budget_id)
        yn.create_transactions(data)
    print("done")



from dataclasses import dataclass
from typing import Optional

from dataclasses_json import dataclass_json


@dataclass
class Transaction(object):
    pass


@dataclass_json
@dataclass
class YnabTransaction(Transaction):
    account_id: str
    date: str
    amount: str
    # payee_id: str #not used
    payee_name: str
    # category_id: str #not used
    memo: str
    import_id: str

    cleared: Optional[str] = None
    approved: bool = False
    flag_color: str = "green"



from typing import List

import requests

from ynab_import.common import settings
from ynab_import.common.models import YnabTransaction
from ynab_import.common.providers import APIProvider


class YnabBudget(APIProvider):
    def __init__(self, api_key: str, budget_id: str) -> None:
        self.api_key = api_key
        self.budget_id = budget_id

    def create_transactions(self, transactions: List[YnabTransaction]) -> int:
        headers = {"Authorization": f"Bearer {self.api_key}"}
        transactions = [t.to_dict() for t in transactions]
        data = {"transactions": transactions}
        response = requests.post(
            settings.YNAB_API_BASE_URL + f"budgets/{self.budget_id}/transactions",
            json=data,
            headers=headers,
        )
        if response.status_code != 200:
            print(response.json())
        response.raise_for_status()
        return response.status_code



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['ynab_import',
 'ynab_import.common',
 'ynab_import.revolut',
 'ynab_import.swedbank',
 'ynab_import.ynab']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1,<8.0',
 'dataclass-csv>=1.1,<2.0',
 'dataclasses-json>=0.3.6,<0.4.0',
 'environ>=1.0,<2.0',
 'environs>=7.1,<8.0']

entry_points = \
{'console_scripts': ['ynab-import = ynab_import.main:main']}

setup_kwargs = {
    'name': 'ynab-import',
    'version': '0.0.1',
    'description': 'Automate transaction import troubles to ynab.',
    'long_description': None,
    'author': 'demonno',
    'author_email': 'demur.nodia@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



class BaseProvider(object):
    pass


class CSVProvider(BaseProvider):
    """Reading data from csv files"""

    def read_csv(self, input_file: str):
        raise NotImplementedError


class APIProvider(BaseProvider):
    """Fetching/Submitting data using APIs"""

    pass
